// Hung En Tsai

package database;

import java.io.*;
import java.text.*;
import java.util.*;
import java.sql.*;

import utils.*;

public final class DBCourse {

    public DBCourse() {
    }

    public int insert() throws SQLException, Exception {

        DBConnection dbCon = DBManager.getInstance().getDBConnectionManager().getConnection();

        String sql = "INSERT INTO DB_Course ( Name, Description, Credit ) VALUES ( ?, ?, ? )";
        PreparedStatement pstmt = dbCon.getConnection().prepareStatement(sql);
        pstmt.setString(1, _Name);
        pstmt.setString(2, _Description);
        pstmt.setString(3, _Credit);
        int courseId = pstmt.executeUpdate();

        DBManager.getInstance().getDBConnectionManager().releaseConnection(dbCon);

        return courseId;
    }

    public void initializeById(int courseId) throws SQLException, Exception {
        DBConnection dbCon = DBManager.getInstance().getDBConnectionManager().getConnection();

        String sql = "SELECT * FROM DB_Course WHERE CourseId = ?";
        PreparedStatement pstmt = dbCon.getConnection().prepareStatement(sql);
        pstmt.setInt(1, courseId);
        ResultSet rs = pstmt.executeQuery();

        if (rs.next()) {
            setCourseId( rs.getInt("CourseId") );
            setName( rs.getString("Name") );
            setDescription( rs.getString("Description") );
            setCredit( rs.getString("Credit") );
        }

        DBManager.getInstance().getDBConnectionManager().releaseConnection(dbCon);
    }

    public void update() throws SQLException, Exception {

        DBConnection dbCon = DBManager.getInstance().getDBConnectionManager().getConnection();

        String sql = "UPDATE DB_Course SET Name = ?, Description = ?, Credit = ? WHERE CourseId = ?";
        PreparedStatement pstmt = dbCon.getConnection().prepareStatement(sql);
        pstmt.setString(1, _Name);
        pstmt.setString(2, _Description);
        pstmt.setString(3, _Credit);
        pstmt.setInt(4, _CourseId);
        pstmt.executeUpdate();

        DBManager.getInstance().getDBConnectionManager().releaseConnection(dbCon);
    }

    public void delete() throws SQLException, Exception {

        DBConnection dbCon = DBManager.getInstance().getDBConnectionManager().getConnection();

        String sql = "DELETE FROM DB_Course WHERE CourseId = ?";
        PreparedStatement pstmt = dbCon.getConnection().prepareStatement(sql);
        pstmt.setInt(1, _CourseId);
        pstmt.executeUpdate();

        DBManager.getInstance().getDBConnectionManager().releaseConnection(dbCon);
    }

    // DBCourse should be smart enough to create table
    static public void createSchema() throws SQLException, Exception {

        DBConnection dbCon = DBManager.getInstance().getDBConnectionManager().getConnection();
        Statement stmt = dbCon.getConnection().createStatement();

        String cmd = "CREATE TABLE DB_Course (\n" +
                     "    CourseId    int IDENTITY,\n" +
                     "    Name        nvarchar(254),\n" +
                     "    Description nvarchar(254),\n" +
                     "    Credit      nvarchar(254),\n" +
                     "    CONSTRAINT db_course_courseid_pk PRIMARY KEY (CourseId)\n" +
                     ");";

        Log.getInstance().msg("DBCourse.createSchema() " + cmd);
        stmt.executeUpdate(cmd);

        DBManager.getInstance().getDBConnectionManager().releaseConnection(dbCon);
    }

    // ----------------------------------------------------------
    // other methods
    // ----------------------------------------------------------

    static public DBCourse getCourseById(int courseId) throws SQLException, Exception {

        DBConnection dbCon = DBManager.getInstance().getDBConnectionManager().getConnection();

        String sql = "SELECT * FROM DB_Course WHERE CourseId = ?";
        PreparedStatement pstmt = dbCon.getConnection().prepareStatement(sql);
        pstmt.setInt(1, courseId);
        ResultSet rs = pstmt.executeQuery();

        DBCourse dbCourse = null;
        if (rs.next()) {
            dbCourse = new DBCourse();
            dbCourse.setCourseId( rs.getInt("CourseId") );
            dbCourse.setName( rs.getString("Name") );
            dbCourse.setDescription( rs.getString("Description") );
            dbCourse.setCredit( rs.getString("Credit") );
        }

        DBManager.getInstance().getDBConnectionManager().releaseConnection(dbCon);

        return dbCourse;
    }

    // should return course ids of all courses
    static public Enumeration getAllCourses() throws SQLException, Exception {

        DBConnection dbCon = DBManager.getInstance().getDBConnectionManager().getConnection();
        Statement stmt = dbCon.getConnection().createStatement();

        String cmd = "SELECT * FROM DB_Course";
        ResultSet rs = stmt.executeQuery(cmd);

        Vector allCourseIds = new Vector();
        while (rs.next()) {
            allCourseIds.addElement( new Integer(rs.getInt("CourseId")) );
        }

        DBManager.getInstance().getDBConnectionManager().releaseConnection(dbCon);

        return allCourseIds.elements();
    }

    // ----------------------------------------------------------
    // member methods
    // ----------------------------------------------------------

    public int getCourseId() {
        return _CourseId;
    }

    public void setCourseId(int courseId) {
        _CourseId = courseId;
    }

    public String getName() {
        return _Name;
    }

    public void setName(String name) {
        _Name = name;
    }

    public String getDescription() {
        return _Description;
    }

    public void setDescription(String description) {
        _Description = description;
    }

    public String getCredit() {
        return _Credit;
    }

    public void setCredit(String credit) {
        _Credit = credit;
    }

    // ----------------------------------------------------------
    // member
    // ----------------------------------------------------------

    // member variables
    private int    _CourseId = -1;
    private String _Name = "";
    private String _Description = "";
    private String _Credit = "";

    // constants
    static public String TABLE_NAME = "DB_Course";
}