// Hung En Tsai

package database;

import java.io.*;
import java.text.*;
import java.util.*;
import java.sql.*;

import utils.*;

public final class DBEnrollment {

    public DBEnrollment() {
    }

    public int insert() throws SQLException, Exception {

        DBConnection dbCon = DBManager.getInstance().getDBConnectionManager().getConnection();

        String sql = "INSERT INTO DB_Enrollment ( SectionId, UserId, Grade ) VALUES ( ?, ?, ? )";
        PreparedStatement pstmt = dbCon.getConnection().prepareStatement(sql);
        pstmt.setInt(1, _SectionId);
        pstmt.setInt(2, _UserId);
        pstmt.setString(3, _Grade);
        int enrollmentId = pstmt.executeUpdate();

        DBManager.getInstance().getDBConnectionManager().releaseConnection(dbCon);

        return enrollmentId;
    }

    public void initializeById(int enrollmentId) throws SQLException, Exception {
        DBConnection dbCon = DBManager.getInstance().getDBConnectionManager().getConnection();

        String sql = "SELECT * FROM DB_Enrollment WHERE EnrollmentId = ?";
        PreparedStatement pstmt = dbCon.getConnection().prepareStatement(sql);
        pstmt.setInt(1, enrollmentId);
        ResultSet rs = pstmt.executeQuery();

        if (rs.next()) {
            setEnrollmentId( rs.getInt("EnrollmentId") );
            setSectionId( rs.getInt("SectionId") );
            setUserId( rs.getInt("UserId") );
            setGrade( rs.getString("Grade") );
        }

        DBManager.getInstance().getDBConnectionManager().releaseConnection(dbCon);
    }

    // To Do: actually, you can only update grade only....
    public void update() throws SQLException, Exception {

        DBConnection dbCon = DBManager.getInstance().getDBConnectionManager().getConnection();

        String sql = "UPDATE DB_Enrollment SET Grade = ? WHERE EnrollmentId = ?";
        PreparedStatement pstmt = dbCon.getConnection().prepareStatement(sql);
        pstmt.setString(1, _Grade);
        pstmt.setInt(2, _EnrollmentId);
        pstmt.executeUpdate();

        DBManager.getInstance().getDBConnectionManager().releaseConnection(dbCon);
    }

    // To Do: actually, you shouldn't be able to delete enrollment after you get grade.
    //        probably should check if enrollment is graded before deleting???
    public void delete() throws SQLException, Exception {

        DBConnection dbCon = DBManager.getInstance().getDBConnectionManager().getConnection();

        String sql = "DELETE FROM DB_Enrollment WHERE EnrollmentId = ?";
        PreparedStatement pstmt = dbCon.getConnection().prepareStatement(sql);
        pstmt.setInt(1, _EnrollmentId);
        pstmt.executeUpdate();

        DBManager.getInstance().getDBConnectionManager().releaseConnection(dbCon);
    }

    // DBEnrollment should be smart enough to create table
    static public void createSchema() throws SQLException, Exception {

        DBConnection dbCon = DBManager.getInstance().getDBConnectionManager().getConnection();
        Statement stmt = dbCon.getConnection().createStatement();

        String cmd = "CREATE TABLE DB_Enrollment (\n" +
                     "    EnrollmentId int IDENTITY,\n" +
                     "    SectionId    int,\n" +
                     "    UserId       int,\n" +
                     "    Grade        nvarchar(254)\n" +
                     "    CONSTRAINT db_enrollment_enrollmentid_pk PRIMARY KEY (EnrollmentId),\n" +
                     "    CONSTRAINT db_enrollment_sectionid_fk FOREIGN KEY (SectionId) REFERENCES DB_Section(SectionId),\n" +
                     "    CONSTRAINT db_enrollment_userid_fk FOREIGN KEY (UserId) REFERENCES DB_User(UserId)\n" +
                     ");";

        Log.getInstance().msg("DBEnrollment.createSchema() " + cmd);
        stmt.executeUpdate(cmd);

        DBManager.getInstance().getDBConnectionManager().releaseConnection(dbCon);
    }

    // --------------------------------------------------------------
    // other methods
    // --------------------------------------------------------------

    static public DBEnrollment getEnrollmentById(int enrollmentId) throws SQLException, Exception {

        DBConnection dbCon = DBManager.getInstance().getDBConnectionManager().getConnection();

        String sql = "SELECT * FROM DB_Enrollment WHERE EnrollmentId = ?";
        PreparedStatement pstmt = dbCon.getConnection().prepareStatement(sql);
        pstmt.setInt(1, enrollmentId);
        ResultSet rs = pstmt.executeQuery();

        DBEnrollment dbEnrollment = null;
        if (rs.next()) {
            dbEnrollment = new DBEnrollment();
            dbEnrollment.setEnrollmentId( rs.getInt("EnrollmentId") );
            dbEnrollment.setSectionId( rs.getInt("SectionId") );
            dbEnrollment.setUserId( rs.getInt("UserId") );
            dbEnrollment.setGrade( rs.getString("Grade") );
        }

        DBManager.getInstance().getDBConnectionManager().releaseConnection(dbCon);

        return dbEnrollment;
    }

    // should return enrollment ids of all enrollments
    static public Enumeration getAllEnrollments() throws SQLException, Exception {

        DBConnection dbCon = DBManager.getInstance().getDBConnectionManager().getConnection();
        Statement stmt = dbCon.getConnection().createStatement();

        String cmd = "SELECT * FROM DB_Enrollment";
        ResultSet rs = stmt.executeQuery(cmd);

        Vector allEnrollmentIds = new Vector();
        while (rs.next()) {
            allEnrollmentIds.addElement( new Integer(rs.getInt("EnrollmentId")) );
        }

        DBManager.getInstance().getDBConnectionManager().releaseConnection(dbCon);

        return allEnrollmentIds.elements();
    }

    static public Enumeration getEnrollmentsByUserId(int userId) throws SQLException, Exception {
        DBConnection dbCon = DBManager.getInstance().getDBConnectionManager().getConnection();

        String sql = "SELECT EnrollmentId FROM DB_Enrollment WHERE UserId = ?";
        PreparedStatement pstmt = dbCon.getConnection().prepareStatement(sql);
        pstmt.setInt(1, userId);
        ResultSet rs = pstmt.executeQuery();

        Vector allSectionIds = new Vector();
        while (rs.next()) {
            allSectionIds.addElement( new Integer(rs.getInt("EnrollmentId")) );
        }

        DBManager.getInstance().getDBConnectionManager().releaseConnection(dbCon);

        return allSectionIds.elements();
    }

    static public Enumeration getEnrollmentsBySectionId(int sectionId) throws SQLException, Exception {
        DBConnection dbCon = DBManager.getInstance().getDBConnectionManager().getConnection();

        String sql = "SELECT * FROM DB_Enrollment WHERE SectionId = ?";
        PreparedStatement pstmt = dbCon.getConnection().prepareStatement(sql);
        pstmt.setInt(1, sectionId);
        ResultSet rs = pstmt.executeQuery();

        Vector allSectionIds = new Vector();
        while (rs.next()) {
            allSectionIds.addElement( new Integer(rs.getInt("EnrollmentId")) );
        }

        DBManager.getInstance().getDBConnectionManager().releaseConnection(dbCon);

        return allSectionIds.elements();
    }

    // ------------------------------------------------------------
    // member methods
    // ------------------------------------------------------------

    public int getEnrollmentId() {
        return _EnrollmentId;
    }

    public void setEnrollmentId(int enrollmentId) {
        _EnrollmentId = enrollmentId;
    }

    public int getSectionId() {
        return _SectionId;
    }

    public void setSectionId(int sectionId) {
        _SectionId = sectionId;
    }

    public int getUserId() {
        return _UserId;
    }

    public void setUserId(int userId) {
        _UserId = userId;
    }

    public String getGrade() {
        return _Grade;
    }

    public void setGrade(String grade) {
        _Grade = grade;
    }

    // ----------------------------------------------------------
    // member
    // ----------------------------------------------------------

    // member variables
    private int    _EnrollmentId = -1;
    private int    _SectionId = -1;
    private int    _UserId = -1;
    private String _Grade = "";

    // constants
    static public String TABLE_NAME = "DB_Enrollment";
}