// Hung En Tsai

package database;

import java.io.*;
import java.text.*;
import java.util.*;
import java.sql.*;

import utils.*;

public final class DBRole {

    public DBRole() {
    }

    public int insert() throws SQLException, Exception {
        // INSERT INTO DB_Role ( Name, Description )
        //     VALUES ( 'Admin', 'administer thing' )

        DBConnection dbCon = DBManager.getInstance().getDBConnectionManager().getConnection();

        String sql = "INSERT INTO DB_Role ( Name, Description ) VALUES ( ?, ? )";
        PreparedStatement pstmt = dbCon.getConnection().prepareStatement(sql);
        pstmt.setString(1, _Name);
        pstmt.setString(2, _Description);
        int roleId = pstmt.executeUpdate();

        DBManager.getInstance().getDBConnectionManager().releaseConnection(dbCon);

        return roleId;
    }

    public void initializeById(int roleId) throws SQLException, Exception {
        DBConnection dbCon = DBManager.getInstance().getDBConnectionManager().getConnection();

        String sql = "SELECT RoleId, Name, Description FROM DB_Role WHERE RoleId = ?";
        PreparedStatement pstmt = dbCon.getConnection().prepareStatement(sql);
        pstmt.setInt(1, roleId);
        ResultSet rs = pstmt.executeQuery();

        if (rs.next()) {
            setRoleId( rs.getInt("RoleId") );
            setName( rs.getString("Name") );
            setDescription( rs.getString("Description") );
        }

        DBManager.getInstance().getDBConnectionManager().releaseConnection(dbCon);
    }

    public void initializeByName(String name) throws SQLException, Exception {

        DBConnection dbCon = DBManager.getInstance().getDBConnectionManager().getConnection();

        String sql = "SELECT RoleId, Name, Description FROM DB_Role WHERE Name = ?";
        PreparedStatement pstmt = dbCon.getConnection().prepareStatement(sql);
        pstmt.setString(1, name);
        ResultSet rs = pstmt.executeQuery();

        if (rs.next()) {
            setRoleId( rs.getInt("RoleId") );
            setName( rs.getString("Name") );
            setDescription( rs.getString("Description") );
        }

        DBManager.getInstance().getDBConnectionManager().releaseConnection(dbCon);
    }

    // To Do: well here, user only is given or remove of a role, never
    // really update it..
    public void update() throws SQLException, Exception {

        DBConnection dbCon = DBManager.getInstance().getDBConnectionManager().getConnection();

        String sql = "UPDATE DB_Role SET Name = ?, Description = ? WHERE RoleId = ?";
        PreparedStatement pstmt = dbCon.getConnection().prepareStatement(sql);
        pstmt.setInt(1, _RoleId);
        pstmt.setString(2, _Name);
        pstmt.setString(3, _Description);
        ResultSet rs = pstmt.executeQuery();
        pstmt.executeUpdate();

        DBManager.getInstance().getDBConnectionManager().releaseConnection(dbCon);
    }

    public void delete() throws SQLException, Exception {

        DBConnection dbCon = DBManager.getInstance().getDBConnectionManager().getConnection();

        String sql = "DELETE FROM DB_Role WHERE RoleId = ?";
        PreparedStatement pstmt = dbCon.getConnection().prepareStatement(sql);
        pstmt.setInt(1, _RoleId);
        pstmt.executeUpdate();

        DBManager.getInstance().getDBConnectionManager().releaseConnection(dbCon);
    }

    // DBRole should be smart enough to create table
    static public void createSchema() throws SQLException, Exception {

        DBConnection dbCon = DBManager.getInstance().getDBConnectionManager().getConnection();
        Statement stmt = dbCon.getConnection().createStatement();

        String cmd = "CREATE TABLE DB_Role (\n" +
                     "    RoleId      int IDENTITY,\n" +
                     "    Name        nvarchar(254),\n" +
                     "    Description nvarchar(254),\n" +
                     "    CONSTRAINT db_role_roleid_pk PRIMARY KEY (RoleId)\n" +
                     ");";

        Log.getInstance().msg("DBRole.createSchema() " + cmd);
        stmt.executeUpdate(cmd);

        DBManager.getInstance().getDBConnectionManager().releaseConnection(dbCon);
    }

    // -----------------------------------------------------------
    // other methods
    // -----------------------------------------------------------

    static public DBRole getRoleById(int roleId) throws SQLException, Exception {

        DBConnection dbCon = DBManager.getInstance().getDBConnectionManager().getConnection();

        String sql = "SELECT RoleId, Name, Description FROM DB_Role WHERE RoleId = ?";
        PreparedStatement pstmt = dbCon.getConnection().prepareStatement(sql);
        pstmt.setInt(1, roleId);
        ResultSet rs = pstmt.executeQuery();

        DBRole dbRole = null;
        if (rs.next()) {
            dbRole = new DBRole();
            dbRole.setRoleId( rs.getInt("RoleId") );
            dbRole.setName( rs.getString("Name") );
            dbRole.setDescription( rs.getString("Description") );
        }

        DBManager.getInstance().getDBConnectionManager().releaseConnection(dbCon);

        return dbRole;
    }

    static public DBRole getRoleByName(String name) throws SQLException, Exception {

        DBConnection dbCon = DBManager.getInstance().getDBConnectionManager().getConnection();

        String sql = "SELECT RoleId, Name, Description FROM DB_Role WHERE Name = ?";
        PreparedStatement pstmt = dbCon.getConnection().prepareStatement(sql);
        pstmt.setString(1, name);
        ResultSet rs = pstmt.executeQuery();

        DBRole dbRole = null;
        if (rs.next()) {
            dbRole = new DBRole();
            dbRole.setRoleId( rs.getInt("RoleId") );
            dbRole.setName( rs.getString("Name") );
            dbRole.setDescription( rs.getString("Description") );
        }

        DBManager.getInstance().getDBConnectionManager().releaseConnection(dbCon);

        return dbRole;
    }

    // should return role ids of all roles
    static public Enumeration getAllRoles() throws SQLException, Exception {

        DBConnection dbCon = DBManager.getInstance().getDBConnectionManager().getConnection();
        Statement stmt = dbCon.getConnection().createStatement();

        String cmd = "SELECT * FROM DB_Role";
        ResultSet rs = stmt.executeQuery(cmd);

        Vector allRoleIds = new Vector();
        while (rs.next()) {
            allRoleIds.addElement( new Integer(rs.getInt("RoleId")) );
        }

        DBManager.getInstance().getDBConnectionManager().releaseConnection(dbCon);

        return allRoleIds.elements();
    }

    // ---------------------------------------------------------
    // member method
    // ---------------------------------------------------------

    public int getRoleId() {
        return _RoleId;
    }

    public void setRoleId(int roleId) {
        _RoleId = roleId;
    }

    public String getName() {
        return _Name;
    }

    public void setName(String name) {
        _Name = name;
    }

    public String getDescription() {
        return _Description;
    }

    public void setDescription(String description) {
        _Description = description;
    }

    // ----------------------------------------------------------
    // member
    // ----------------------------------------------------------

    // member variables
    private int    _RoleId = -1;
    private String _Name = "";
    private String _Description = "";

    // constants
    static public String TABLE_NAME = "DB_Role";
}