// Hung En Tsai

package database;

import java.io.*;
import java.text.*;
import java.util.*;
import java.sql.*;

import utils.*;

public final class DBSection {

    public DBSection() {
    }

    public int insert() throws SQLException, Exception {

        DBConnection dbCon = DBManager.getInstance().getDBConnectionManager().getConnection();

        String sql = "INSERT INTO DB_Section ( CourseId, Section, Year, Season, Time, Room, UserId ) VALUES ( ?, ?, ?, ?, ?, ?, ? )";
        PreparedStatement pstmt = dbCon.getConnection().prepareStatement(sql);
        pstmt.setInt(1, _CourseId);
        pstmt.setString(2, _Section);
        pstmt.setString(3, _Year);
        pstmt.setString(4, _Season);
        pstmt.setString(5, _Time);
        pstmt.setString(6, _Room);
        pstmt.setInt(7, _UserId);
        int sectionId = pstmt.executeUpdate();

        DBManager.getInstance().getDBConnectionManager().releaseConnection(dbCon);

        return sectionId;
    }

    public void initializeById(int sectionId) throws SQLException, Exception {

        DBConnection dbCon = DBManager.getInstance().getDBConnectionManager().getConnection();

        String sql = "SELECT * FROM DB_Section WHERE SectionId = ?";
        PreparedStatement pstmt = dbCon.getConnection().prepareStatement(sql);
        pstmt.setInt(1, sectionId);
        ResultSet rs = pstmt.executeQuery();

        if (rs.next()) {
            setSectionId( rs.getInt("SectionId") );
            setCourseId( rs.getInt("CourseId") );
            setSection( rs.getString("Section") );
            setYear( rs.getString("Year") );
            setSeason( rs.getString("Season") );
            setTime( rs.getString("Time") );
            setRoom( rs.getString("Room") );
            setUserId( rs.getInt("UserId") );
        }

        DBManager.getInstance().getDBConnectionManager().releaseConnection(dbCon);
    }

    public void update() throws SQLException, Exception {

        DBConnection dbCon = DBManager.getInstance().getDBConnectionManager().getConnection();

        String sql = "UPDATE DB_Section SET CourseId = ?, Section = ?, Year = ?, Season = ?, Time = ?, Room = ?, UserId = ? WHERE SectionId = ?";
        PreparedStatement pstmt = dbCon.getConnection().prepareStatement(sql);
        pstmt.setInt(1, _CourseId);
        pstmt.setString(2, _Section);
        pstmt.setString(3, _Year);
        pstmt.setString(4, _Season);
        pstmt.setString(5, _Time);
        pstmt.setString(6, _Room);
        pstmt.setInt(7, _UserId);
        pstmt.setInt(8, _SectionId);
        pstmt.executeUpdate();

        DBManager.getInstance().getDBConnectionManager().releaseConnection(dbCon);
    }

    // To Do: probably shouldn't allow deletion
    public void delete() throws SQLException, Exception {

        DBConnection dbCon = DBManager.getInstance().getDBConnectionManager().getConnection();

        String sql = "DELETE FROM DB_Section WHERE SectionId = ?";
        PreparedStatement pstmt = dbCon.getConnection().prepareStatement(sql);
        pstmt.setInt(1, _SectionId);
        pstmt.executeUpdate();

        DBManager.getInstance().getDBConnectionManager().releaseConnection(dbCon);
    }

    // DBSection should be smart enough to create table
    static public void createSchema() throws SQLException, Exception {

        DBConnection dbCon = DBManager.getInstance().getDBConnectionManager().getConnection();
        Statement stmt = dbCon.getConnection().createStatement();

        String cmd = "CREATE TABLE DB_Section (\n" +
                     "    SectionId  int IDENTITY,\n" +
                     "    CourseId   int,\n" +
                     "    Section    nvarchar(254),\n" +
                     "    Year       nvarchar(254),\n" +
                     "    Season     nvarchar(254),\n" +
                     "    Time       nvarchar(254),\n" +
                     "    Room       nvarchar(254),\n" +
                     "    UserId     int,\n" +
                     "    CONSTRAINT db_section_sectionid_pk PRIMARY KEY (SectionId),\n" +
                     "    CONSTRAINT db_section_userid_fk FOREIGN KEY (UserId) REFERENCES DB_User(UserId),\n" +
                     "    CONSTRAINT db_section_courseid_fk FOREIGN KEY (CourseId) REFERENCES DB_COURSE(CourseId)\n" +
                     ");";

        Log.getInstance().msg("DBSection.createSchema() " + cmd);
        stmt.executeUpdate(cmd);

        DBManager.getInstance().getDBConnectionManager().releaseConnection(dbCon);
    }

    // -------------------------------------------------------------
    // other methods
    // -------------------------------------------------------------

    static public DBSection getSectionById(int sectionId) throws SQLException, Exception {

        DBConnection dbCon = DBManager.getInstance().getDBConnectionManager().getConnection();

        String sql = "SELECT * FROM DB_Section WHERE SectionId = ?";
        PreparedStatement pstmt = dbCon.getConnection().prepareStatement(sql);
        pstmt.setInt(1, sectionId);
        ResultSet rs = pstmt.executeQuery();

        DBSection dbSection = null;
        if (rs.next()) {
            dbSection = new DBSection();
            dbSection.setSectionId( rs.getInt("SectionId") );
            dbSection.setCourseId( rs.getInt("CourseId") );
            dbSection.setSection( rs.getString("Section") );
            dbSection.setYear( rs.getString("Year") );
            dbSection.setSeason( rs.getString("Season") );
            dbSection.setTime( rs.getString("Time") );
            dbSection.setRoom( rs.getString("Room") );
            dbSection.setUserId( rs.getInt("UserId") );
        }

        DBManager.getInstance().getDBConnectionManager().releaseConnection(dbCon);

        return dbSection;
    }

    // should return section ids of all sections
    static public Enumeration getAllSections() throws SQLException, Exception {

        DBConnection dbCon = DBManager.getInstance().getDBConnectionManager().getConnection();
        Statement stmt = dbCon.getConnection().createStatement();

        String cmd = "SELECT * FROM DB_Section";
        ResultSet rs = stmt.executeQuery(cmd);

        Vector allSectionIds = new Vector();
        while (rs.next()) {
            allSectionIds.addElement( new Integer(rs.getInt("SectionId")) );
        }

        DBManager.getInstance().getDBConnectionManager().releaseConnection(dbCon);

        return allSectionIds.elements();
    }

    // To Do: this is more for user with instructor role that teaches this
    // section
    static public Enumeration getSectionsByUserId(int userId) throws SQLException, Exception {

        DBConnection dbCon = DBManager.getInstance().getDBConnectionManager().getConnection();

        String sql = "SELECT SectionId FROM DB_Section WHERE UserId = ?";
        PreparedStatement pstmt = dbCon.getConnection().prepareStatement(sql);
        pstmt.setInt(1, userId);
        ResultSet rs = pstmt.executeQuery();

        Vector allSectionIds = new Vector();
        while (rs.next()) {
            allSectionIds.addElement( new Integer(rs.getInt("SectionId")) );
        }

        DBManager.getInstance().getDBConnectionManager().releaseConnection(dbCon);

        return allSectionIds.elements();
    }

    // -------------------------------------------------------------
    // member methods
    // -------------------------------------------------------------

    public int getSectionId() {
        return _SectionId;
    }

    public void setSectionId(int sectionId) {
        _SectionId = sectionId;
    }

    public int getCourseId() {
        return _CourseId;
    }

    public void setCourseId(int courseId) {
        _CourseId = courseId;
    }

    public String getSection() {
        return _Section;
    }

    public void setSection(String section) {
        _Section = section;
    }

    public String getYear() {
        return _Year;
    }

    public void setYear(String year) {
        _Year = year;
    }

    public String getSeason() {
        return _Season;
    }

    public void setSeason(String season) {
        _Season = season;
    }

    public String getTime() {
        return _Time;
    }

    public void setTime(String time) {
        _Time = time;
    }

    public String getRoom() {
        return _Room;
    }

    public void setRoom(String room) {
        _Room = room;
    }

    public int getUserId() {
        return _UserId;
    }

    public void setUserId(int userId) {
        _UserId = userId;
    }

    // -------------------------------------------------------------
    // other
    // -------------------------------------------------------------

    // member variables
    private int    _SectionId = -1;
    private int    _CourseId = -1;
    private String _Section = "";
    private String _Year = "";
    private String _Season = "";
    private String _Time = "";
    private String _Room = "";
    private int    _UserId = -1;

    // constants
    static public String TABLE_NAME = "DB_Section";
}