// Hung En Tsai

package database;

import java.io.*;
import java.text.*;
import java.util.*;
import java.sql.*;

import utils.*;

public final class DBUser {

    public DBUser() {
    }

    // Statement: is interface with two method
    //   executeQuery(String sql) returns ResultSet
    //   executeUpdate(String sql) return int
    //
    // PrepareStatement: is interface with
    //   executeQuery() returns ResultSet
    //   executeUpdate() return int
    public int insert() throws SQLException, Exception {

        DBConnection dbCon = DBManager.getInstance().getDBConnectionManager().getConnection();

        String sql = "INSERT INTO DB_User ( FirstName, LastName, Login, Password, Email, RoleId ) VALUES ( ?, ?, ?, ?, ?, ? )";
        PreparedStatement pstmt = dbCon.getConnection().prepareStatement(sql);
        pstmt.setString(1, _FirstName);
        pstmt.setString(2, _LastName);
        pstmt.setString(3, _Login);
        pstmt.setString(4, _Password);
        pstmt.setString(5, _Email);
        pstmt.setInt(6, _RoleId);
        int userId = pstmt.executeUpdate();

        DBManager.getInstance().getDBConnectionManager().releaseConnection(dbCon);

        return userId;
    }

    public void initializeById(int userId) throws SQLException, Exception {

        DBConnection dbCon = DBManager.getInstance().getDBConnectionManager().getConnection();

        /*
        String sql = "SELECT * FROM DB_User WHERE UserId = ?";
        PreparedStatement pstmt = dbCon.getConnection().prepareStatement(sql);
        pstmt.setInt(1, userId);
        ResultSet rs = pstmt.executeQuery();
        */

        String call_string = "{CALL sp_user_lookupById(?)}";
        CallableStatement cs = dbCon.getConnection().prepareCall(call_string);
        cs.setInt(1, userId);
        ResultSet rs = cs.executeQuery();

        if (rs.next()) {
            setUserId( rs.getInt("UserId") );
            setFirstName( rs.getString("FirstName") );
            setLastName( rs.getString("LastName") );
            setLogin( rs.getString("Login") );
            setPassword( rs.getString("Password") );
            setEmail( rs.getString("Email") );
            setRoleId( rs.getInt("RoleId") );
        } else {
            throw new Exception("user not found");
        }

        DBManager.getInstance().getDBConnectionManager().releaseConnection(dbCon);
    }

    public void initializeByLogin(String login) throws SQLException, Exception {

        DBConnection dbCon = DBManager.getInstance().getDBConnectionManager().getConnection();

        /*
        String sql = "SELECT * FROM DB_User WHERE Login = ?";
        PreparedStatement pstmt = dbCon.getConnection().prepareStatement(sql);
        pstmt.setString(1, login);
        ResultSet rs = pstmt.executeQuery();
        */

        String call_string = "{CALL sp_user_lookupByLogin(?)}";
        CallableStatement cs = dbCon.getConnection().prepareCall(call_string);
        cs.setString(1, login);
        ResultSet rs = cs.executeQuery();

        if (rs.next()) {
            setUserId( rs.getInt("UserId") );
            setFirstName( rs.getString("FirstName") );
            setLastName( rs.getString("LastName") );
            setLogin( rs.getString("Login") );
            setPassword( rs.getString("Password") );
            setEmail( rs.getString("Email") );
            setRoleId( rs.getInt("RoleId") );
        }

        DBManager.getInstance().getDBConnectionManager().releaseConnection(dbCon);
    }

    public void update() throws SQLException, Exception {

        DBConnection dbCon = DBManager.getInstance().getDBConnectionManager().getConnection();

        String sql = "UPDATE DB_User SET FirstName = ?, LastName = ?, Password = ?, Email = ?, RoleId = ? WHERE UserId = ?";
        PreparedStatement pstmt = dbCon.getConnection().prepareStatement(sql);
        pstmt.setString(1, _FirstName);
        pstmt.setString(2, _LastName);
        pstmt.setString(3, _Password);
        pstmt.setString(4, _Email);
        pstmt.setInt(5, _RoleId);
        pstmt.setInt(6, _UserId);
        pstmt.executeUpdate();

        DBManager.getInstance().getDBConnectionManager().releaseConnection(dbCon);
    }

    public void delete() throws SQLException, Exception {

        DBConnection dbCon = DBManager.getInstance().getDBConnectionManager().getConnection();

        String sql = "DELETE FROM DB_User WHERE UserId = ?";
        PreparedStatement pstmt = dbCon.getConnection().prepareStatement(sql);
        pstmt.setInt(1, _UserId);
        pstmt.executeUpdate();

        DBManager.getInstance().getDBConnectionManager().releaseConnection(dbCon);
    }

    // DBUser should be smart enough to create table
    static public void createSchema() throws SQLException, Exception {

        Log.getInstance().msg("DBUser.createSchema: enter");

        DBConnection dbCon = DBManager.getInstance().getDBConnectionManager().getConnection();
        Statement stmt = dbCon.getConnection().createStatement();

        String cmd = "CREATE TABLE DB_User (\n" +
                     "    UserId    int IDENTITY,\n" +
                     "    FirstName nvarchar(254),\n" +
                     "    LastName  nvarchar(254),\n" +
                     "    Login     nvarchar(254) UNIQUE,\n" +
                     "    Password  nvarchar(254),\n" +
                     "    Email     nvarchar(254),\n" +
                     "    RoleId    int,\n" +
                     "    CONSTRAINT db_user_userid_pk PRIMARY KEY (UserId),\n" +
                     "    CONSTRAINT db_user_roleid_fk FOREIGN KEY (RoleId) REFERENCES DB_Role(RoleId)\n" +
                     ");";

        Log.getInstance().msg("DBUser.createSchema() " + cmd);
        stmt.executeUpdate(cmd);

        DBManager.getInstance().getDBConnectionManager().releaseConnection(dbCon);
    }

    // ------------------------------------------------------------------
    // other methods
    // ------------------------------------------------------------------

    static public DBUser getUserById(int userId) throws SQLException, Exception {

        DBConnection dbCon = DBManager.getInstance().getDBConnectionManager().getConnection();

        /*
        String sql = "SELECT * FROM DB_User WHERE UserId = ?";
        PreparedStatement pstmt = dbCon.getConnection().prepareStatement(sql);
        pstmt.setInt(1, userId);
        ResultSet rs = pstmt.executeQuery();
        */

        String call_string = "{CALL sp_user_lookupById(?)}";
        CallableStatement cs = dbCon.getConnection().prepareCall(call_string);
        cs.setInt(1, userId);
        ResultSet rs = cs.executeQuery();

        DBUser dbUser = null;
        if (rs.next()) {
            dbUser = new DBUser();
            dbUser.setUserId( rs.getInt("UserId") );
            dbUser.setFirstName( rs.getString("FirstName") );
            dbUser.setLastName( rs.getString("LastName") );
            dbUser.setLogin( rs.getString("Login") );
            dbUser.setPassword( rs.getString("Password") );
            dbUser.setEmail( rs.getString("Email") );
            dbUser.setRoleId( rs.getInt("RoleId") );
        }

        DBManager.getInstance().getDBConnectionManager().releaseConnection(dbCon);

        return dbUser;
    }

    static public Enumeration getUsersByRoleId(int roleId) throws SQLException, Exception {

        DBConnection dbCon = DBManager.getInstance().getDBConnectionManager().getConnection();

        String sql = "SELECT * FROM DB_User WHERE RoleId = ?";
        PreparedStatement pstmt = dbCon.getConnection().prepareStatement(sql);
        pstmt.setInt(1, roleId);
        ResultSet rs = pstmt.executeQuery();

        Vector userIds = new Vector();
        while (rs.next()) {
            userIds.addElement( new Integer(rs.getInt("UserId")) );
        }

        DBManager.getInstance().getDBConnectionManager().releaseConnection(dbCon);

        return userIds.elements();
    }

    // should return user ids of all users
    static public Enumeration getAllUsers() throws SQLException, Exception {

        DBConnection dbCon = DBManager.getInstance().getDBConnectionManager().getConnection();
        Statement stmt = dbCon.getConnection().createStatement();

        String cmd = "SELECT * FROM DB_User";
        ResultSet rs = stmt.executeQuery(cmd);

        Vector allUserIds = new Vector();
        while (rs.next()) {
            allUserIds.addElement( new Integer(rs.getInt("UserId")) );
        }

        DBManager.getInstance().getDBConnectionManager().releaseConnection(dbCon);

        return allUserIds.elements();
    }

    static public Enumeration getAllUsersSortedBy(int sortKey) throws SQLException, Exception {

        DBConnection dbCon = DBManager.getInstance().getDBConnectionManager().getConnection();
        Statement stmt = dbCon.getConnection().createStatement();

        String cmd = "SELECT * FROM DB_User";

        switch (sortKey)
        {
            case KEY_LOGIN:
                cmd = "SELECT * FROM DB_User ORDER BY Login";
                break;
            case KEY_FIRSTNAME:
                cmd = "SELECT * FROM DB_User ORDER BY FirstName";
                break;
            case KEY_LASTNAME:
                cmd = "SELECT * FROM DB_User ORDER BY LastName";
                break;
            case KEY_EMAIL:
                cmd = "SELECT * FROM DB_User ORDER BY Email";
                break;
            default:
        }

        ResultSet rs = stmt.executeQuery(cmd);

        Vector allUserIds = new Vector();
        while (rs.next()) {
            allUserIds.addElement( new Integer(rs.getInt("UserId")) );
        }

        DBManager.getInstance().getDBConnectionManager().releaseConnection(dbCon);

        return allUserIds.elements();
    }

    // ----------------------------------------------------------------
    // member method
    // ----------------------------------------------------------------

    public int getUserId() {
        return _UserId;
    }

    public void setUserId(int userId) {
        _UserId = userId;
    }

    public String getFirstName() {
        return _FirstName;
    }

    public void setFirstName(String firstName) {
        _FirstName = firstName;
    }

    public String getLastName() {
        return _LastName;
    }

    public void setLastName(String lastName) {
        _LastName = lastName;
    }

    public String getLogin() {
        return _Login;
    }

    public void setLogin(String login) {
        _Login = login;
    }

    public String getPassword() {
        return _Password;
    }

    public void setPassword(String password) {
        _Password = password;
    }

    public String getEmail() {
        return _Email;
    }

    public void setEmail(String email) {
        _Email = email;
    }

    public int getRoleId() {
        return _RoleId;
    }

    public void setRoleId(int roleId) {
        _RoleId = roleId;
    }

    // ----------------------------------------------------------
    // member
    // ----------------------------------------------------------

    // sort key
    public final static int KEY_LOGIN     = 0;
    public final static int KEY_FIRSTNAME = 1;
    public final static int KEY_LASTNAME  = 2;
    public final static int KEY_EMAIL     = 3;

    // member variables
    private int    _UserId = -1;
    private String _FirstName = "";
    private String _LastName = "";
    private String _Login = "";
    private String _Password = "";
    private String _Email = "";
    private int    _RoleId = -1;

    // constants
    static public String TABLE_NAME = "DB_User";
}